@@ -41,9 +41,9 @@ occuMS <- function(detformulas, psiformulas, phiformulas=NULL, data,
 
   #Index guide used to organize p values
   guide <- matrix(NA,nrow=S,ncol=S)
-  guide <- lower.tri(guide,diag=T)
+  guide <- lower.tri(guide,diag=TRUE)
   guide[,1] <- FALSE
-  guide <- which(guide,arr.ind=T)
+  guide <- which(guide,arr.ind=TRUE)
   #----------------------------------------------------------------------------
 
   #Likelihood function in R----------------------------------------------------
