@@ -161,7 +161,10 @@ setClass("unmarkedFrameDSO",
 #Convert covs provided as list of matrices/dfs to data frame
 covsToDF <- function(covs, name, obsNum, numSites){
   if(!inherits(covs, "list")) return(covs)
-
+  
+  if(is.null(names(covs)) | any(is.na(names(covs))) | any(names(covs)=="")){
+    stop("All elements of list provided to ", name, " argument must be named", call.=FALSE)
+  }
   lapply(covs, function(x){
     if(!inherits(x, c("matrix", "data.frame")))
       stop(paste("At least one element of", name, "is not a matrix or data frame."))
@@ -1133,9 +1136,10 @@ setMethod("[", c("unmarkedFrame", "numeric", "missing", "missing"),
     if (!is.null(obsCovs)) {
         R <- obsNum(x)
         .site <- rep(1:M, each = R)
-        obsCovs <- ldply(i, function(site) {
-            subset(obsCovs, .site == site)
-            })
+        oc <- lapply(i, function(ind){
+         obsCovs[.site==ind,,drop=FALSE]
+        })
+        obsCovs <- do.call(rbind, oc)
         }
     umf <- x
     umf@y <- y
@@ -1191,17 +1195,20 @@ setMethod("[", c("unmarkedFrame","list", "missing", "missing"),
     if (m != length(i)) stop("list length must be same as number of sites.")
     siteCovs <- siteCovs(x)
     y <- cbind(.site=1:m, getY(x))
-    obsCovs <- as.data.frame(cbind(.site=rep(1:m, each=R), obsCovs(x)))
+    obsCovs <- obsCovs(x)
+    site_idx <- rep(1:m, each=R)
+    stopifnot(length(site_idx) == nrow(obsCovs))
 
-    obsCovs <- ddply(obsCovs, ~.site, function(df) {
-        site <- df$.site[1]
-        obs <- i[[site]]
-        if (length(obs) > R)
-            stop("All elements of list must be less than or equal to R.")
-        obs <- c(obs, rep(NA, R-length(obs)))
-        df[obs,]
-        })
-    obsCovs$.site <- NULL
+    oc <- lapply(1:m, function(ind){
+      df <- obsCovs[site_idx==ind,,drop=FALSE]
+      obs <- i[[ind]]
+      if (length(obs) > R)
+        stop("All elements of list must be less than or equal to R.")
+      obs <- c(obs, rep(NA, R-length(obs)))
+      df[obs,,drop=FALSE]
+    })
+    obsCovs <- do.call(rbind, oc)
+    rownames(obsCovs) <- NULL
 
     y <- apply(y, 1, function(row) {
         site <- row[1]
@@ -1235,9 +1242,10 @@ setMethod("[", c("unmarkedFrameOccuMulti", "numeric", "missing", "missing"),
     if (!is.null(obsCovs)) {
         R <- obsNum(x)
         .site <- rep(1:M, each = R)
-        obsCovs <- ldply(i, function(site) {
-            subset(obsCovs, .site == site)
-            })
+        oc <- lapply(i, function(ind){
+         obsCovs[.site==ind,,drop=FALSE]
+        })
+        obsCovs <- do.call(rbind, oc)
         }
     umf <- x
     umf@y <- ylist[[1]]
@@ -1310,9 +1318,10 @@ setMethod("[", c("unmarkedMultFrame", "numeric", "missing", "missing"),
     if (!is.null(obsCovs)) {
         R <- obsNum(x)
         .site <- rep(1:M, each = obsNum(x)) #NULL     ## testing
-        obsCovs <- ldply(i, function(site) {
-            subset(obsCovs, .site == site)
-            })
+        oc <- lapply(i, function(ind){
+         obsCovs[.site==ind,,drop=FALSE]
+        })
+        obsCovs <- do.call(rbind, oc)
         }
     u <- unmarkedMultFrame(y=matrix(y, ncol=ncol(oldy)),
                            siteCovs=siteCovs,
